@@ -48,11 +48,13 @@ annotations = lib.annotate_features(sample.features_df)
 import os
 import json
 from typing import Optional, Union, List, Dict, Any, TYPE_CHECKING
-import warnings
 
 import polars as pl
 import pyopenms as oms
 
+# Import logger
+from masster.logger import MassterLogger
+
 if TYPE_CHECKING:
     import pandas as pd
 
@@ -250,6 +252,7 @@ class Lib:
     def __init__(self):
         """Initialize an empty Lib instance."""
         self.lib_df = None
+        self.logger = MassterLogger("lib", level="INFO")
         self._initialize_empty_dataframe()
     
     def _initialize_empty_dataframe(self):
@@ -257,7 +260,7 @@ class Lib:
         self.lib_df = pl.DataFrame({
             "lib_uid": pl.Series([], dtype=pl.Int64),
             "cmpd_uid": pl.Series([], dtype=pl.Int64),
-            "source_id": pl.Series([], dtype=pl.Utf8),
+            "lib_source": pl.Series([], dtype=pl.Utf8),
             "name": pl.Series([], dtype=pl.Utf8),
             "shortname": pl.Series([], dtype=pl.Utf8),
             "class": pl.Series([], dtype=pl.Utf8),
@@ -268,13 +271,14 @@ class Lib:
             "iso": pl.Series([], dtype=pl.Int64),
             "adduct": pl.Series([], dtype=pl.Utf8),
             "probability": pl.Series([], dtype=pl.Float64),
+            "stars": pl.Series([], dtype=pl.Int64),
             "m": pl.Series([], dtype=pl.Float64),
             "z": pl.Series([], dtype=pl.Int8),
             "mz": pl.Series([], dtype=pl.Float64),
             "rt": pl.Series([], dtype=pl.Float64),
             "quant_group": pl.Series([], dtype=pl.Int64),
-            "db_id": pl.Series([], dtype=pl.Utf8),
             "db": pl.Series([], dtype=pl.Utf8),
+            "db_id": pl.Series([], dtype=pl.Utf8),
         })
     
     def _get_adducts(self, 
@@ -448,6 +452,127 @@ class Lib:
         
         return adducts_df
     
+    def _neutralize_charged_formula(self, formula: str) -> Optional[str]:
+        """
+        Neutralize charged formulas by adjusting hydrogen count.
+        
+        Handles formulas ending with charge indicators:
+        - +, 2+, 3+: Remove H, 2H, 3H respectively
+        - -, 2-, 3-: Add H, 2H, 3H respectively
+        
+        Args:
+            formula: Molecular formula string (e.g., "C6H12O6+", "C5H14NO+", "C4H5O4-")
+            
+        Returns:
+            Neutralized formula string, or None if formula is invalid/empty
+        """
+        if not formula or not isinstance(formula, str):
+            return None
+            
+        formula = formula.strip()
+        if not formula:
+            return None
+        
+        # Check for charge indicator at the end
+        charge = 0
+        original_formula = formula
+        
+        # Check for positive charges (+, 2+, 3+, etc.)
+        if formula.endswith('+'):
+            # Handle multi-charge like "2+", "3+"
+            if len(formula) >= 2 and formula[-2].isdigit():
+                charge = int(formula[-2])
+                formula = formula[:-2]
+            else:
+                charge = 1
+                formula = formula[:-1]
+        
+        # Check for negative charges (-, 2-, 3-, etc.)
+        elif formula.endswith('-'):
+            # Handle multi-charge like "2-", "3-"
+            if len(formula) >= 2 and formula[-2].isdigit():
+                charge = -int(formula[-2])
+                formula = formula[:-2]
+            else:
+                charge = -1
+                formula = formula[:-1]
+        
+        # If no charge found, return original formula
+        if charge == 0:
+            return original_formula
+        
+        # Neutralize by adjusting hydrogen count
+        # Positive charge: remove H atoms (formula had extra H)
+        # Negative charge: add H atoms (formula was missing H)
+        
+        try:
+            # Use OpenMS to parse the formula and get element counts
+            empirical_formula = oms.EmpiricalFormula(formula)
+            
+            # Convert to string and parse to get hydrogen count
+            formula_str = str(empirical_formula)
+            
+            # Simple regex-based approach to adjust H count
+            import re
+            
+            # Find hydrogen in the formula (H followed by optional number)
+            h_match = re.search(r'H(\d*)', formula_str)
+            
+            if h_match:
+                h_count_str = h_match.group(1)
+                current_h = int(h_count_str) if h_count_str else 1
+            else:
+                # No hydrogen in formula
+                current_h = 0
+            
+            # Calculate new hydrogen count
+            # For positive charge (+), we remove H
+            # For negative charge (-), we add H
+            new_h = current_h - charge
+            
+            if new_h < 0:
+                self.logger.trace(f"Cannot neutralize formula '{original_formula}': would result in negative hydrogen count")
+                return None
+            
+            # Reconstruct formula with new hydrogen count
+            if h_match:
+                if new_h == 0:
+                    # Remove H entirely
+                    neutralized = re.sub(r'H\d*', '', formula_str)
+                elif new_h == 1:
+                    # H without number
+                    neutralized = re.sub(r'H\d*', 'H', formula_str)
+                else:
+                    # H with number
+                    neutralized = re.sub(r'H\d*', f'H{new_h}', formula_str)
+            else:
+                # Need to add hydrogen
+                if new_h > 0:
+                    # Add H at the beginning (or after C if present)
+                    if formula_str.startswith('C'):
+                        # Find where C ends
+                        c_match = re.match(r'C(\d*)', formula_str)
+                        if c_match:
+                            after_c = len(c_match.group(0))
+                            h_str = 'H' if new_h == 1 else f'H{new_h}'
+                            neutralized = formula_str[:after_c] + h_str + formula_str[after_c:]
+                        else:
+                            neutralized = f'H{new_h}' + formula_str
+                    else:
+                        h_str = 'H' if new_h == 1 else f'H{new_h}'
+                        neutralized = h_str + formula_str
+                else:
+                    neutralized = formula_str
+            
+            # Clean up any empty elements
+            neutralized = neutralized.strip()
+            
+            return neutralized
+            
+        except Exception as e:
+            self.logger.trace(f"Error neutralizing formula '{original_formula}': {e}")
+            return None
+    
     def _calculate_accurate_mass(self, formula: str) -> Optional[float]:
         """
         Calculate the accurate mass for a molecular formula using PyOpenMS.
@@ -482,7 +607,7 @@ class Lib:
         for pattern_check in invalid_patterns:
             try:
                 if pattern_check(formula):
-                    warnings.warn(f"Skipping obviously invalid formula: '{formula}'")
+                    self.logger.trace(f"Skipping obviously invalid formula: '{formula}'")
                     return None
             except Exception:
                 # If pattern checking fails, continue to PyOpenMS parsing
@@ -492,7 +617,7 @@ class Lib:
             empirical_formula = oms.EmpiricalFormula(formula)
             return empirical_formula.getMonoWeight()
         except Exception as e:
-            warnings.warn(f"Error calculating accurate mass for formula '{formula}': {e}")
+            self.logger.trace(f"Error calculating accurate mass for formula '{formula}': {e}")
             return None
     
     def _generate_adduct_variants(self, 
@@ -547,7 +672,7 @@ class Lib:
             variant = {
                 "lib_uid": counter,
                 "cmpd_uid": compound_data.get("cmpd_uid", None),
-                "source_id": compound_data.get("source_id", None),
+                "lib_source": compound_data.get("lib_source", None),
                 "name": compound_data.get("name", ""),
                 "shortname": compound_data.get("shortname", ""),
                 "class": compound_data.get("class", ""),
@@ -558,6 +683,7 @@ class Lib:
                 "iso": 0,  # Default to zero
                 "adduct": adduct_name,
                 "probability": probability,
+                "stars": compound_data.get("stars", 0),
                 "m": adducted_mass,
                 "z": charge,
                 "mz": mz,
@@ -623,6 +749,20 @@ class Lib:
         for row in df.iter_rows(named=True):
             total_compounds += 1
             
+            # Get formula and skip if empty/None
+            formula = row[column_mapping["formula"]]
+            if not formula or not isinstance(formula, str) or not formula.strip():
+                skipped_compounds += 1
+                continue
+            
+            # Neutralize charged formulas
+            formula = formula.strip()
+            neutralized_formula = self._neutralize_charged_formula(formula)
+            if neutralized_formula is None:
+                # Skip if neutralization failed
+                skipped_compounds += 1
+                continue
+            
             # Extract compound data
             # assign a compound-level uid so all adducts share the same cmpd_uid
             compound_level_uid = cmpd_id_counter
@@ -635,10 +775,11 @@ class Lib:
                 "smiles": row.get(column_mapping.get("smiles", ""), ""),
                 "inchi": row.get(column_mapping.get("inchi", ""), ""),
                 "inchikey": row.get(column_mapping.get("inchikey", ""), ""),
-                "formula": row[column_mapping["formula"]],
+                "formula": neutralized_formula,
                 "rt": self._safe_float_conversion(row.get(column_mapping.get("rt", ""), None)),
                 "db_id": row.get(column_mapping.get("db_id", ""), None),
                 "db": row.get(column_mapping.get("db", ""), None),
+                "stars": self._safe_int_conversion(row.get(column_mapping.get("stars", ""), 0)),
                 "cmpd_uid": compound_level_uid,
             }
             
@@ -679,12 +820,12 @@ class Lib:
                 self.lib_df = new_lib_df
                 
             #successful_compounds = total_compounds - skipped_compounds
-            print(f"Imported {len(all_variants)} library entries from {csvfile}")
+            self.logger.info(f"Imported {len(all_variants)} library entries from {csvfile}")
             #print(f"Processed {total_compounds} compounds: {successful_compounds} successful, {skipped_compounds} skipped due to invalid formulas")
         else:
-            print(f"No valid compounds found in {csvfile}")
+            self.logger.warning(f"No valid compounds found in {csvfile}")
             if skipped_compounds > 0:
-                print(f"All {total_compounds} compounds were skipped due to invalid formulas")
+                self.logger.warning(f"All {total_compounds} compounds were skipped due to invalid formulas")
     
     def import_json(self, 
                    jsonfile: str, 
@@ -755,7 +896,7 @@ class Lib:
         source_file = json_data.get("source_file", "unknown")
         record_count = json_data.get("record_count", len(data))
         
-        print(f"Loading JSON library: version {version}, source: {source_file}, records: {record_count}")
+        self.logger.info(f"Loading embedded library: {source_file}, version {version}, records: {record_count}")
         
         # Process each compound
         all_variants = []
@@ -772,8 +913,17 @@ class Lib:
                 skipped_compounds += 1
                 continue
             
-            formula = compound_record.get("formula", "")
-            if not formula or not isinstance(formula, str):
+            # Get formula and skip if empty/None
+            formula = compound_record.get("formula", compound_record.get("Formula", ""))
+            if not formula or not isinstance(formula, str) or not formula.strip():
+                skipped_compounds += 1
+                continue
+            
+            # Neutralize charged formulas
+            formula = formula.strip()
+            neutralized_formula = self._neutralize_charged_formula(formula)
+            if neutralized_formula is None:
+                # Skip if neutralization failed
                 skipped_compounds += 1
                 continue
             
@@ -788,10 +938,11 @@ class Lib:
                 "smiles": compound_record.get("smiles", compound_record.get("SMILES", "")),
                 "inchi": compound_record.get("inchi", compound_record.get("InChI", "")),
                 "inchikey": compound_record.get("inchikey", compound_record.get("InChIKey", "")),
-                "formula": formula,
+                "formula": neutralized_formula,
                 "rt": self._safe_float_conversion(compound_record.get("rt", compound_record.get("RT", None))),
                 "db_id": compound_record.get("db_id", compound_record.get("database_id", None)),
                 "db": compound_record.get("db", compound_record.get("database", None)),
+                "stars": self._safe_int_conversion(compound_record.get("stars", 0)),
                 "cmpd_uid": compound_level_uid,
             }
             
@@ -816,11 +967,11 @@ class Lib:
             else:
                 self.lib_df = new_lib_df
                 
-            print(f"Imported {len(all_variants)} library entries from {jsonfile}")
+            self.logger.info(f"Imported {len(all_variants)} library entries from {jsonfile}")
         else:
-            print(f"No valid compounds found in {jsonfile}")
+            self.logger.warning(f"No valid compounds found in {jsonfile}")
             if skipped_compounds > 0:
-                print(f"All {total_compounds} compounds were skipped due to invalid formulas")
+                self.logger.warning(f"All {total_compounds} compounds were skipped due to invalid formulas")
     
     def _map_csv_columns(self, columns: List[str]) -> Dict[str, str]:
         """
@@ -889,6 +1040,12 @@ class Lib:
                 mapping["db"] = columns[columns_lower.index(db_variant)]
                 break
         
+        # Stars mapping
+        for stars_variant in ["stars", "star", "confidence"]:
+            if stars_variant in columns_lower:
+                mapping["stars"] = columns[columns_lower.index(stars_variant)]
+                break
+        
         return mapping
     
     def _safe_float_conversion(self, value: Any) -> Optional[float]:
@@ -908,6 +1065,23 @@ class Lib:
         except (ValueError, TypeError):
             return None
     
+    def _safe_int_conversion(self, value: Any) -> Optional[int]:
+        """
+        Safely convert a value to int, returning None if conversion fails.
+        
+        Args:
+            value: Value to convert
+            
+        Returns:
+            Int value or None
+        """
+        if value is None or value == "":
+            return None
+        try:
+            return int(value)
+        except (ValueError, TypeError):
+            return None
+    
     def annotate_features(self, 
                          features_df: Union[pl.DataFrame, "pd.DataFrame"],
                          mz_tolerance: float = 0.01,
@@ -978,7 +1152,7 @@ class Lib:
                     "feature_rt": feature_rt,
                     "lib_uid": match_row["lib_uid"],
                     "cmpd_uid": match_row.get("cmpd_uid"),
-                    "source_id": match_row.get("source_id"),
+                    "lib_source": match_row.get("lib_source"),
                     "name": match_row["name"],
                     "shortname": match_row["shortname"],
                     "class": match_row["class"],
@@ -991,6 +1165,7 @@ class Lib:
                     "lib_mz": match_row["mz"],
                     "lib_rt": match_row["rt"],
                     "quant_group": match_row.get("quant_group"),
+                    "stars": match_row.get("stars"),
                     "delta_mz": abs(feature_mz - match_row["mz"]),
                     "delta_rt": abs(feature_rt - match_row["rt"]) if feature_rt is not None and match_row["rt"] is not None else None,
                 }
@@ -1011,6 +1186,500 @@ class Lib:
         adducts_df = self._get_adducts(polarity=polarity, min_probability=0.0)
         return adducts_df.select("name").to_series().to_list()
     
+    def compare(
+        self,
+        file: Optional[str] = None,
+        action: str = "intersect",
+        on: str = "inchikey14",
+        keep_none: bool = False,
+    ) -> Optional[pl.DataFrame]:
+        """
+        Compare current library with another library and perform actions based on matching compounds.
+        
+        This method loads a reference library and compares it with the current library based on a 
+        specified comparison field (formula, inchikey, inchikey14, or name). It then performs 
+        various actions on rows based on whether they match the reference library.
+        
+        Args:
+            file: Path to reference library file (CSV or JSON). If None, no comparison is performed.
+            action: Action to perform based on comparison results:
+                - 'reset_stars' or 'reset_star': Set stars=0 for rows NOT in reference library
+                - 'add_stars' or 'add_star': Increment stars by 1 for rows in reference library
+                - 'delete': Remove rows that ARE in reference library
+                - 'filter' or 'delete_others': Remove rows that are NOT in reference library
+                - 'intersect': Return DataFrame of lib_df rows that ARE in reference library
+                - 'difference': Return DataFrame of lib_df rows that are NOT in reference library
+                - 'missing': Return DataFrame of reference library rows not matched in lib_df
+            on: Field to compare on. Valid values: 'formula', 'inchikey', 'inchikey14', 'name'
+            keep_none: If True, treat None/null values as valid matches (keep them regardless)
+        
+        Returns:
+            None for modification actions (modifies self.lib_df in place)
+            pl.DataFrame for query actions ('intersect', 'difference', 'missing')
+        
+        Raises:
+            ValueError: If invalid 'on' or 'action' value provided
+            FileNotFoundError: If file doesn't exist
+        
+        Example:
+            >>> lib = Lib()
+            >>> lib.import_csv("my_compounds.csv")
+            >>> # Reset stars for compounds not in reference library
+            >>> lib.compare(file="reference_lib.csv", action="reset_stars", on="inchikey14")
+            >>> # Filter to keep only compounds in reference library
+            >>> lib.compare(file="reference_lib.csv", action="filter", on="name")
+            >>> # Get compounds that are in both libraries
+            >>> common = lib.compare(file="reference_lib.csv", action="intersect", on="inchikey14")
+            >>> # Get compounds missing from reference
+            >>> missing = lib.compare(file="reference_lib.csv", action="missing", on="formula")
+        """
+        # Validate inputs
+        valid_on_values = ['formula', 'inchikey', 'inchikey14', 'name']
+        if on not in valid_on_values:
+            raise ValueError(f"Invalid 'on' value: {on}. Must be one of {valid_on_values}")
+        
+        valid_actions = ['reset_stars', 'reset_star', 'add_stars', 'add_star', 'delete', 'filter', 'delete_others', 'intersect', 'difference', 'missing']
+        if action not in valid_actions:
+            raise ValueError(f"Invalid action: {action}. Must be one of {valid_actions}")
+        
+        # Check if current library is empty
+        if self.lib_df is None or self.lib_df.is_empty():
+            self.logger.warning("Current library is empty. No comparison performed.")
+            return
+        
+        # If no file provided, nothing to compare
+        if file is None:
+            self.logger.warning("No reference file provided. No comparison performed.")
+            return
+        
+        # Check if file exists
+        if not os.path.exists(file):
+            raise FileNotFoundError(f"Reference library file not found: {file}")
+        
+        # Load reference library
+        new_lib = Lib()
+        if file.lower().endswith('.json'):
+            new_lib.import_json(file)
+        elif file.lower().endswith('.csv'):
+            new_lib.import_csv(file)
+        else:
+            # Try CSV by default
+            new_lib.import_csv(file)
+        
+        if new_lib.lib_df is None or new_lib.lib_df.is_empty():
+            self.logger.warning(f"Reference library from {file} is empty. No comparison performed.")
+            return
+        
+        # Extract comparison values from reference library
+        if on == 'inchikey14':
+            # Use first 14 characters of inchikey
+            if 'inchikey' not in new_lib.lib_df.columns:
+                raise ValueError("Reference library does not have 'inchikey' column for inchikey14 comparison")
+            valid_values = (
+                new_lib.lib_df
+                .select(pl.col('inchikey').str.slice(0, 14).alias('inchikey14'))
+                .unique()
+                .drop_nulls()
+                .to_series()
+                .to_list()
+            )
+        else:
+            # Use the column directly
+            if on not in new_lib.lib_df.columns:
+                raise ValueError(f"Reference library does not have '{on}' column")
+            valid_values = (
+                new_lib.lib_df
+                .select(on)
+                .unique()
+                .drop_nulls()
+                .to_series()
+                .to_list()
+            )
+        
+        # Check if comparison column exists in current library
+        if on == 'inchikey14':
+            if 'inchikey' not in self.lib_df.columns:
+                raise ValueError("Current library does not have 'inchikey' column for inchikey14 comparison")
+            # Create temporary column for comparison
+            comparison_col = pl.col('inchikey').str.slice(0, 14)
+        else:
+            if on not in self.lib_df.columns:
+                raise ValueError(f"Current library does not have '{on}' column")
+            comparison_col = pl.col(on)
+        
+        # Create mask for rows that match reference library
+        is_valid = comparison_col.is_in(valid_values)
+        
+        # Handle None values if keep_none is True
+        if keep_none:
+            is_none = comparison_col.is_null()
+            is_valid = is_valid | is_none
+        
+        # Apply action based on the mask
+        if action in ['reset_stars', 'reset_star']:
+            # Set stars=0 for rows NOT in reference library
+            self.lib_df = self.lib_df.with_columns(
+                pl.when(~is_valid)
+                .then(pl.lit(0))
+                .otherwise(pl.col('stars'))
+                .alias('stars')
+            )
+        
+        elif action in ['add_stars', 'add_star']:
+            # Increment stars by 1 for rows in reference library
+            self.lib_df = self.lib_df.with_columns(
+                pl.when(is_valid)
+                .then(pl.col('stars') + 1)
+                .otherwise(pl.col('stars'))
+                .alias('stars')
+            )
+        
+        elif action == 'delete':
+            # Remove rows that ARE in reference library
+            self.lib_df = self.lib_df.filter(~is_valid)
+        
+        elif action in ['filter', 'delete_others']:
+            # Remove rows that are NOT in reference library
+            self.lib_df = self.lib_df.filter(is_valid)
+        
+        elif action == 'intersect':
+            # Return rows of lib_df that ARE in reference library
+            return self.lib_df.filter(is_valid)
+        
+        elif action == 'difference':
+            # Return rows of lib_df that are NOT in reference library
+            return self.lib_df.filter(~is_valid)
+        
+        elif action == 'missing':
+            # Return rows of reference library that are NOT matched in lib_df
+            # Get the comparison values from current lib_df
+            if on == 'inchikey14':
+                current_values = (
+                    self.lib_df
+                    .select(pl.col('inchikey').str.slice(0, 14).alias('inchikey14'))
+                    .unique()
+                    .drop_nulls()
+                    .to_series()
+                    .to_list()
+                )
+                # Filter new_lib for rows not in current library
+                missing_df = new_lib.lib_df.filter(
+                    ~pl.col('inchikey').str.slice(0, 14).is_in(current_values)
+                )
+                # Deduplicate by inchikey14 (keep first occurrence)
+                missing_df = missing_df.unique(subset=['inchikey'], keep='first', maintain_order=True)
+            else:
+                current_values = (
+                    self.lib_df
+                    .select(on)
+                    .unique()
+                    .drop_nulls()
+                    .to_series()
+                    .to_list()
+                )
+                # Filter new_lib for rows not in current library
+                missing_df = new_lib.lib_df.filter(~pl.col(on).is_in(current_values))
+                # Deduplicate by the comparison column (keep first occurrence)
+                missing_df = missing_df.unique(subset=[on], keep='first', maintain_order=True)
+            
+            return missing_df
+        
+        return None
+    
+    def lib_select(
+        self,
+        uid=None,
+        cmpd_uid=None,
+        lib_source=None,
+        name=None,
+        shortname=None,
+        class_=None,
+        formula=None,
+        inchikey=None,
+        inchikey14=None,
+        adduct=None,
+        iso=None,
+        mz=None,
+        rt=None,
+        probability=None,
+        stars=None,
+        z=None,
+        quant_group=None,
+        db=None,
+    ) -> pl.DataFrame:
+        """
+        Select library entries based on specified criteria and return the filtered DataFrame.
+        
+        This method filters lib_df based on various criteria for compound properties,
+        returning a Polars DataFrame with matching entries.
+        
+        Args:
+            uid: lib_uid filter (list of UIDs, tuple for range, or None for all)
+            cmpd_uid: compound UID filter (list, tuple for range, or single value)
+            lib_source: library source filter (str for exact match, list for multiple sources)
+            name: compound name filter using regex (str for pattern, list for multiple patterns with OR)
+            shortname: short name filter using regex (str for pattern, list for multiple patterns)
+            class_: compound class filter using regex (str for pattern, list for multiple patterns)
+            formula: molecular formula filter (str for exact match, list for multiple formulas)
+            inchikey: InChIKey filter (str for exact match, list for multiple keys)
+            inchikey14: InChIKey first 14 chars filter (str for exact match, list for multiple)
+            adduct: adduct filter (str for exact match, list for multiple adducts)
+            iso: isotope number filter (tuple for range, single value for exact match)
+            mz: m/z range filter (tuple for range, single value for minimum)
+            rt: retention time range filter (tuple for range, single value for minimum)
+            probability: adduct probability filter (tuple for range, single value for minimum)
+            stars: stars rating filter (tuple for range, single value for exact match)
+            z: charge filter (int for exact match, list for multiple charges)
+            quant_group: quantification group filter (int for exact match, list for multiple groups)
+            db: database filter (str for exact match, list for multiple databases)
+        
+        Returns:
+            pl.DataFrame: Filtered library DataFrame
+        
+        Examples:
+            >>> lib = Lib()
+            >>> lib.import_csv("compounds.csv")
+            >>> # Select by m/z range
+            >>> selected = lib.lib_select(mz=(100, 500))
+            >>> # Select by name pattern
+            >>> selected = lib.lib_select(name="glucose")
+            >>> # Select by multiple criteria
+            >>> selected = lib.lib_select(adduct="[M+H]+", stars=(3, 5))
+        """
+        if self.lib_df is None or self.lib_df.is_empty():
+            self.logger.warning("Library is empty.")
+            return pl.DataFrame()
+        
+        lib = self.lib_df.clone()
+        
+        # Filter by lib_uid
+        if uid is not None:
+            if isinstance(uid, tuple) and len(uid) == 2:
+                min_uid, max_uid = uid
+                lib = lib.filter((pl.col("lib_uid") >= min_uid) & (pl.col("lib_uid") <= max_uid))
+            elif isinstance(uid, list):
+                lib = lib.filter(pl.col("lib_uid").is_in(uid))
+            else:
+                lib = lib.filter(pl.col("lib_uid") == uid)
+        
+        # Filter by cmpd_uid
+        if cmpd_uid is not None:
+            if isinstance(cmpd_uid, tuple) and len(cmpd_uid) == 2:
+                min_uid, max_uid = cmpd_uid
+                lib = lib.filter((pl.col("cmpd_uid") >= min_uid) & (pl.col("cmpd_uid") <= max_uid))
+            elif isinstance(cmpd_uid, list):
+                lib = lib.filter(pl.col("cmpd_uid").is_in(cmpd_uid))
+            else:
+                lib = lib.filter(pl.col("cmpd_uid") == cmpd_uid)
+        
+        # Filter by lib_source
+        if lib_source is not None:
+            if isinstance(lib_source, list):
+                lib = lib.filter(pl.col("lib_source").is_in(lib_source))
+            else:
+                lib = lib.filter(pl.col("lib_source") == lib_source)
+        
+        # Filter by name (regex)
+        if name is not None:
+            if isinstance(name, list):
+                # Combine multiple patterns with OR
+                pattern = "|".join(name)
+                lib = lib.filter(pl.col("name").str.contains(pattern, literal=False))
+            else:
+                lib = lib.filter(pl.col("name").str.contains(name, literal=False))
+        
+        # Filter by shortname (regex)
+        if shortname is not None:
+            if isinstance(shortname, list):
+                pattern = "|".join(shortname)
+                lib = lib.filter(pl.col("shortname").str.contains(pattern, literal=False))
+            else:
+                lib = lib.filter(pl.col("shortname").str.contains(shortname, literal=False))
+        
+        # Filter by class (regex)
+        if class_ is not None:
+            if isinstance(class_, list):
+                pattern = "|".join(class_)
+                lib = lib.filter(pl.col("class").str.contains(pattern, literal=False))
+            else:
+                lib = lib.filter(pl.col("class").str.contains(class_, literal=False))
+        
+        # Filter by formula
+        if formula is not None:
+            if isinstance(formula, list):
+                lib = lib.filter(pl.col("formula").is_in(formula))
+            else:
+                lib = lib.filter(pl.col("formula") == formula)
+        
+        # Filter by inchikey
+        if inchikey is not None:
+            if isinstance(inchikey, list):
+                lib = lib.filter(pl.col("inchikey").is_in(inchikey))
+            else:
+                lib = lib.filter(pl.col("inchikey") == inchikey)
+        
+        # Filter by inchikey14 (first 14 characters)
+        if inchikey14 is not None:
+            if isinstance(inchikey14, list):
+                lib = lib.filter(pl.col("inchikey").str.slice(0, 14).is_in(inchikey14))
+            else:
+                lib = lib.filter(pl.col("inchikey").str.slice(0, 14) == inchikey14)
+        
+        # Filter by adduct
+        if adduct is not None:
+            if isinstance(adduct, list):
+                lib = lib.filter(pl.col("adduct").is_in(adduct))
+            else:
+                lib = lib.filter(pl.col("adduct") == adduct)
+        
+        # Filter by iso
+        if iso is not None:
+            if isinstance(iso, tuple) and len(iso) == 2:
+                min_iso, max_iso = iso
+                lib = lib.filter((pl.col("iso") >= min_iso) & (pl.col("iso") <= max_iso))
+            else:
+                lib = lib.filter(pl.col("iso") == iso)
+        
+        # Filter by mz
+        if mz is not None:
+            if isinstance(mz, tuple) and len(mz) == 2:
+                min_mz, max_mz = mz
+                lib = lib.filter((pl.col("mz") >= min_mz) & (pl.col("mz") <= max_mz))
+            else:
+                lib = lib.filter(pl.col("mz") >= mz)
+        
+        # Filter by rt
+        if rt is not None:
+            if isinstance(rt, tuple) and len(rt) == 2:
+                min_rt, max_rt = rt
+                lib = lib.filter((pl.col("rt") >= min_rt) & (pl.col("rt") <= max_rt))
+            else:
+                lib = lib.filter(pl.col("rt") >= rt)
+        
+        # Filter by probability
+        if probability is not None:
+            if isinstance(probability, tuple) and len(probability) == 2:
+                min_prob, max_prob = probability
+                lib = lib.filter((pl.col("probability") >= min_prob) & (pl.col("probability") <= max_prob))
+            else:
+                lib = lib.filter(pl.col("probability") >= probability)
+        
+        # Filter by stars
+        if stars is not None:
+            if isinstance(stars, tuple) and len(stars) == 2:
+                min_stars, max_stars = stars
+                lib = lib.filter((pl.col("stars") >= min_stars) & (pl.col("stars") <= max_stars))
+            else:
+                lib = lib.filter(pl.col("stars") == stars)
+        
+        # Filter by charge
+        if z is not None:
+            if isinstance(z, list):
+                lib = lib.filter(pl.col("z").is_in(z))
+            else:
+                lib = lib.filter(pl.col("z") == z)
+        
+        # Filter by quant_group
+        if quant_group is not None:
+            if isinstance(quant_group, list):
+                lib = lib.filter(pl.col("quant_group").is_in(quant_group))
+            else:
+                lib = lib.filter(pl.col("quant_group") == quant_group)
+        
+        # Filter by db
+        if db is not None:
+            if isinstance(db, list):
+                lib = lib.filter(pl.col("db").is_in(db))
+            else:
+                lib = lib.filter(pl.col("db") == db)
+        
+        return lib
+    
+    def lib_filter(self, entries):
+        """
+        Keep only the specified library entries and delete all others (modifies lib_df in place).
+        
+        This method filters lib_df to keep only specified entries, removing all others.
+        Similar to features_filter() but for library entries.
+        
+        Args:
+            entries: Can be one of the following:
+                - list: List of lib_uid values to keep
+                - pl.DataFrame or pd.DataFrame: DataFrame with 'lib_uid' column - extracts unique values to keep
+                - str: Special action:
+                    * 'delete_identified': Delete all library entries that appear in id_df
+                    * 'delete_orphans': Delete all library entries that do NOT appear in id_df
+        
+        Returns:
+            None (modifies self.lib_df in place)
+        
+        Raises:
+            ValueError: If entries type is invalid or required data is missing
+        
+        Examples:
+            >>> lib = Lib()
+            >>> lib.import_csv("compounds.csv")
+            >>> # Keep only specific UIDs
+            >>> lib.lib_filter([0, 1, 2, 10, 15])
+            >>> # Keep only entries from a filtered DataFrame
+            >>> selected = lib.lib_select(stars=(4, 5))
+            >>> lib.lib_filter(selected)
+        """
+        if self.lib_df is None or self.lib_df.is_empty():
+            self.logger.warning("Library is empty.")
+            return
+        
+        if entries is None:
+            self.logger.warning("No entries specified to keep.")
+            return
+        
+        original_count = len(self.lib_df)
+        
+        # Handle string actions
+        if isinstance(entries, str):
+            # For string actions, we need id_df
+            # Since Lib class doesn't have id_df, we can't support these actions here
+            # These will be implemented in Sample and Study wrappers
+            raise ValueError(
+                f"String action '{entries}' not supported in Lib class. "
+                "Use Sample.lib_filter() or Study.lib_filter() instead."
+            )
+        
+        # Handle DataFrame input
+        elif hasattr(entries, 'columns'):
+            # Check if it's a polars or pandas DataFrame
+            if 'lib_uid' in entries.columns:
+                # Extract unique lib_uid values
+                if hasattr(entries, 'select'):  # Polars
+                    lib_uids_to_keep = entries.select('lib_uid').unique().to_series().to_list()
+                else:  # Pandas
+                    lib_uids_to_keep = entries['lib_uid'].unique().tolist()
+            else:
+                raise ValueError("DataFrame must contain 'lib_uid' column")
+        
+        # Handle list input
+        elif isinstance(entries, (list, tuple)):
+            lib_uids_to_keep = list(entries)
+        
+        else:
+            raise ValueError(
+                "entries must be a list of lib_uid values, a DataFrame with 'lib_uid' column, "
+                "or a string action ('delete_identified' or 'delete_orphans')"
+            )
+        
+        if not lib_uids_to_keep:
+            self.logger.warning("No valid lib_uid values provided to keep.")
+            return
+        
+        # Filter lib_df to keep only specified entries
+        self.lib_df = self.lib_df.filter(pl.col("lib_uid").is_in(lib_uids_to_keep))
+        
+        kept_count = len(self.lib_df)
+        deleted_count = original_count - kept_count
+        
+        # Note: We don't log here since Lib doesn't have a logger by default
+        # Logging will be done in Sample/Study wrappers
+    
     def __len__(self) -> int:
         """Return number of library entries."""
         return len(self.lib_df) if self.lib_df is not None else 0
