@@ -137,7 +137,7 @@ def _get_adducts(self, adducts_list: list = None, **kwargs):
                         "formatted_name": formatted_name,
                         "total_mass_shift": spec["mass_shift"] * multiplier,
                         "total_charge": total_charge,
-                        "combined_probability": spec["probability"] ** multiplier,
+                        "combined_probability": (spec["probability"] ** multiplier) / 2.0,
                         "complexity": multiplier,
                     },
                 )
