@@ -114,6 +114,9 @@ def lib_load(
                 "Could not import masster.lib.lib.Lib - required for CSV/JSON loading",
             )
 
+        # Resolve library name to full path (handles embedded libraries like 'yeast', 'ecoli', etc.)
+        lib_source = _resolve_library_path(lib_source)
+
         lib_obj = Lib()
 
         # Determine file type by extension
