@@ -491,10 +491,12 @@ def import_tima(
             except (ValueError, TypeError):
                 pass
 
-        # Get class and clean NaN values
-        class_value = row.get("label_classyfire", None)
-        if class_value is None or (isinstance(class_value, str) and class_value.upper() == "NAN"):
-            class_value = None
+        # Get class and clean NaN values (only if label_classyfire column exists)
+        class_value = None
+        if "label_classyfire" in row:
+            class_value = row.get("label_classyfire", None)
+            if class_value is None or (isinstance(class_value, str) and class_value.upper() == "NAN"):
+                class_value = None
 
         lib_entry = {
             "lib_uid": row["lib_uid"],
@@ -712,13 +714,17 @@ def import_tima(
     id_columns = {
         "id_top_name": best_ids.select("feature_uid", pl.col("label_compound_short").alias("label_compound")),
         "id_top_adduct": best_ids.select("feature_uid", "adduct"),
-        "id_top_class": best_ids.select("feature_uid", "label_classyfire"),
         "id_top_score": best_ids.select("feature_uid", pl.col("score").round(3).alias("score")),
         "id_source": best_ids.select("feature_uid", "matcher"),
     }
+    
+    # Only add id_top_class if label_classyfire column exists
+    if "label_classyfire" in best_ids.columns:
+        id_columns["id_top_class"] = best_ids.select("feature_uid", "label_classyfire")
 
-    # Initialize identification columns in features_df if they don't exist
-    for col_name in id_columns.keys():
+    # Initialize all expected identification columns in features_df if they don't exist
+    expected_id_columns = ["id_top_name", "id_top_adduct", "id_top_class", "id_top_score", "id_source"]
+    for col_name in expected_id_columns:
         if col_name not in self.features_df.columns:
             if col_name == "id_top_score":
                 self.features_df = self.features_df.with_columns(pl.lit(None, dtype=pl.Float64).alias(col_name))
