@@ -63,6 +63,7 @@ def load(
     ondisk=False,
     type=None,
     label=None,
+    sample_idx=None,
 ):
     """
     Load file content from a specified filename.
@@ -73,13 +74,15 @@ def load(
         type (str, optional): Specifies the type of file. If provided and set to 'ztscan' (case-insensitive), the type
                                 attribute will be adjusted accordingly. Defaults to None.
         label (Any, optional): An optional label to associate with the loaded file. Defaults to None.
+        sample_idx (int, optional): For WIFF files with multiple samples, specifies which sample to load (0-indexed).
+                                    Defaults to None (loads the last sample if multiple exist).
     Raises:
         FileNotFoundError: If the file specified by filename does not exist.
         ValueError: If the file extension is not one of the supported types (.mzML, .wiff, or .raw).
     Notes:
         The function determines the appropriate internal loading mechanism based on the file extension:
             - ".mzml": Calls _load_mzML(filename)
-            - ".wiff": Calls _load_wiff(filename)
+            - ".wiff": Calls _load_wiff(filename, sample_idx)
             - ".raw": Calls _load_raw(filename)
         After loading, the type attribute is set to 'dda', unless the optional 'type' parameter is provided as 'ztscan',
         in which case it is updated to 'ztscan'. The label attribute is updated if a label is provided.
@@ -89,14 +92,14 @@ def load(
         filename = self.file_path
     filename = os.path.abspath(filename)
     if not os.path.exists(filename):
-        raise FileNotFoundError("Filename not valid. Provide a valid file path.")
+        raise FileNotFoundError("File not found.")
     self.ondisk = ondisk
 
     # check if file is mzML
     if filename.lower().endswith(".mzml"):
         _load_mzML(self, filename)
     elif filename.lower().endswith(".wiff") or filename.lower().endswith(".wiff2"):
-        _load_wiff(self, filename)
+        _load_wiff(self, filename, sample_idx=sample_idx)
     elif filename.lower().endswith(".raw"):
         _load_raw(self, filename)
     elif filename.lower().endswith(".sample5"):
@@ -120,6 +123,7 @@ def load_noms1(
     ondisk=False,
     type=None,
     label=None,
+    sample_idx=None,
 ):
     """
     Optimized load method that skips loading ms1_df for better performance.
@@ -133,6 +137,8 @@ def load_noms1(
         ondisk (bool, optional): Whether to load on-disk or in-memory. Defaults to False.
         type (str, optional): Override file type detection. Can be "ztscan". Defaults to None.
         label (str, optional): Override sample label. Defaults to None.
+        sample_idx (int, optional): For WIFF files with multiple samples, specifies which sample to load (0-indexed).
+                                    Defaults to None (loads the last sample if multiple exist).
 
     Raises:
         FileNotFoundError: If the specified file doesn't exist.
@@ -155,7 +161,7 @@ def load_noms1(
     if filename.lower().endswith(".mzml"):
         _load_mzML(self, filename)
     elif filename.lower().endswith(".wiff") or filename.lower().endswith(".wiff2"):
-        _load_wiff(self, filename)
+        _load_wiff(self, filename, sample_idx=sample_idx)
     elif filename.lower().endswith(".raw"):
         _load_raw(self, filename)
     elif filename.lower().endswith(".sample5"):
@@ -580,11 +586,10 @@ def _load_raw(
 def _load_wiff(
     self,
     filename=None,
+    sample_idx=None,
 ):
     # Use masster's own implementation first
-    from masster.sample.sciex import SciexWiffData as MassterSciexWiffData
-
-    SciexWiffDataClass = MassterSciexWiffData
+    from masster.sample.sciex import SciexWiffData, count_samples
 
     if not filename:
         raise ValueError("Filename must be provided.")
@@ -594,7 +599,7 @@ def _load_wiff(
     if not os.path.exists(filename):
         raise FileNotFoundError(f"File {filename} not found.")
 
-    raw_data = SciexWiffDataClass(centroided=False)
+    raw_data = SciexWiffData(centroided=False)
     raw_data.keep_k_peaks_per_spec = self.parameters.max_points_per_spectrum
 
     if not filename.endswith(".wiff"):
@@ -602,6 +607,20 @@ def _load_wiff(
     if not os.path.exists(filename):
         raise FileNotFoundError(f"File {filename} not found.")
 
+    # Check number of samples in the WIFF file
+    num_samples = count_samples(filename)
+    if num_samples == 0:
+        raise ValueError("No samples found in the WIFF file.")
+    if sample_idx is None:
+        sample_idx = num_samples - 1  # Default to last sample
+    if sample_idx > num_samples - 1:
+        raise ValueError(f"sample_idx {sample_idx} out of range. File contains {num_samples} samples.")    
+    if num_samples > 1:
+        self.logger.warning(
+            f"WIFF file contains {num_samples} samples. Only loading sample index {sample_idx}."
+        )
+        raw_data.sample_id = sample_idx
+
     self.logger.info(f"Loading {filename}")
     raw_data.import_raw(filename)
 
