@@ -1474,7 +1474,9 @@ def export_mztab(self, filename=None, title=None, description=None, include_mgf=
             adduct_list[idx - 1],  # adduct_ion
             isotopomer,
             safe_str(row.get("mz", "null")),  # exp_mass_to_charge
-            safe_str(row.get("charge", "null")),
+            safe_str(
+                f"{abs(row.get('charge', 1))}+" if row.get('charge', 1) >= 0 else f"{abs(row.get('charge', 1))}-"
+            ) if row.get("charge") is not None and row.get("charge") != "null" else "null",  # charge with sign
             safe_str(row.get("rt", "null")),  # retention_time_in_seconds
             safe_str(row.get("rt_start", "null")),
             safe_str(row.get("rt_end", "null")),
@@ -1591,7 +1593,9 @@ def export_mztab(self, filename=None, title=None, description=None, include_mgf=
                         "null",  # derivatized_form
                         adduct_ion,
                         exp_mz,
-                        safe_str(feature_row.get("charge", "1")),
+                        safe_str(
+                            f"{abs(feature_row.get('charge', 1))}+" if feature_row.get('charge', 1) >= 0 else f"{abs(feature_row.get('charge', 1))}-"
+                        ) if feature_row.get("charge") is not None and feature_row.get("charge") != "null" else "1+",  # charge with sign
                         theoretical_mz,
                         spectra_ref,
                         id_method,
