@@ -669,6 +669,14 @@ class Spectrum:
                     s.inty = savgol_filter(s.inty, smooth, 2)
             except:  # noqa: E722
                 pass
+        
+        # Sort data by m/z before peak finding to ensure correct m/z assignment
+        if not np.all(np.diff(s.mz) >= 0):
+            sort_idx = np.argsort(s.mz)
+            s.mz = s.mz[sort_idx]
+            s.inty = s.inty[sort_idx]
+            not_smothed_inty = not_smothed_inty[sort_idx]
+        
         if prominence is not None and prominence < 0 and s.bl is not None:
             prominence = s.bl
         with warnings.catch_warnings():
@@ -685,26 +693,41 @@ class Spectrum:
             s.width = props["widths"]
             s.prominence = props["prominences"]
         if refine:
-            prof_mz = s.mz
-            prof_inty = s.inty
+            # CRITICAL: Use .copy() to prevent modifying array during iteration
+            prof_mz = s.mz.copy()
+            prof_inty = s.inty.copy()
+            prof_not_smoothed = not_smothed_inty.copy()
+            
+            # Optimized vectorized refinement using searchsorted for neighbor finding
+            mz_window = 0.05  # Da
+            
             for idx in peaks:
-                idxs = np.arange(idx - 2, idx + 3)
-                if idxs[0] < 0 or idxs[-1] >= len(prof_mz):
+                # Use binary search to find neighbors efficiently (O(log n) instead of O(n))
+                mz_center = prof_mz[idx]
+                left_idx = np.searchsorted(prof_mz, mz_center - mz_window, side='left')
+                right_idx = np.searchsorted(prof_mz, mz_center + mz_window, side='right')
+                
+                # Skip if not enough neighbors
+                if right_idx - left_idx < 3:
                     continue
-                s.mz[idx] = np.average(prof_mz[idxs], weights=prof_inty[idxs] ** 3 + 1)
-                inty_smoothed = np.max(prof_inty[idxs])
-                inty_not_smoothed = np.max(not_smothed_inty[idxs])
-                s.inty[idx] = np.max([inty_smoothed, inty_not_smoothed])
+                
+                # Use the neighbors for refinement
+                neighbor_mz = prof_mz[left_idx:right_idx]
+                neighbor_inty = prof_inty[left_idx:right_idx]
+                neighbor_not_smoothed = prof_not_smoothed[left_idx:right_idx]
+                
+                # Vectorized weight calculation and max operations
+                weights = neighbor_inty ** 3 + 1
+                s.mz[idx] = np.average(neighbor_mz, weights=weights)
+                s.inty[idx] = max(np.max(neighbor_inty), np.max(neighbor_not_smoothed))
             s.mz = s.mz[peaks]
             s.inty = s.inty[peaks]
             s.history_add("c[SLMR]")
-            s.history_add("c[SLMR]")
             s.centroided = True
         else:
             s.mz = s.mz[peaks]
             s.inty = props["peak_heights"]
             s.history_add("c[SLM]")
-            s.history_add("c[SLM]")
             s.centroided = True
         if prominence is not None:
             mask = s.prominence >= prominence
@@ -713,7 +736,6 @@ class Spectrum:
             s.width = s.width[mask]
             s.prominence = s.prominence[mask]
             s.history_add("f[PRO]")
-            s.history_add("f[PRO]")
         return s
 
     def deisotope(self, mz_tol: float = 0.02, ratio_max: float = 1.5) -> Spectrum:
@@ -743,7 +765,6 @@ class Spectrum:
         if self_c.label is not None:
             self_c.label = self_c.label + " deiso."
         self_c.history_add("f[iso]")
-        self_c.history_add("f[iso]")
         return self_c
 
     # TODO externalize params
@@ -1091,7 +1112,7 @@ def combine_peaks(
     )
 
 
-# TODO externalize params
+
 def plot_spectra(
     spectra: list[Spectrum],
     labels: list[str] | None = None,
@@ -1105,6 +1126,7 @@ def plot_spectra(
     filename: str | None = None,
     colorby: str | None = None,
     ylog: bool = False,
+    stacked: bool = False,
 ) -> None:
     """
     Plot multiple mass spectrometry spectra on a single Bokeh figure.
@@ -1132,6 +1154,8 @@ def plot_spectra(
                                     have an appropriate extension, the plot is simply displayed.
         ylog (bool, optional): If True, the y-axis is set to a logarithmic scale. Default is False.
         colorby (str, optional): If provided, the color of each spectrum is determined by this attribute.
+        stacked (bool, optional): If True, each spectrum is plotted in a separate vertically stacked subplot with
+                                    synchronized x-axes for zooming. Default is False.
 
     Returns:
         None
@@ -1163,179 +1187,384 @@ def plot_spectra(
         for i in range(num_plots)
     ]
 
-    p = figure(
-        width=width,
-        height=height,
-        title=title,
-    )
-
-    for spec_idx, spec in enumerate(spectra):
-        try:
-            label = f"Spectrum {spec_idx}"
-            if spec.label is not None:
-                label = spec.label
-            if labels is not None and len(labels) >= num_plots:
-                label = labels[spec_idx]
-
-            mcvalues = None
-            mcolors = ["black"] * len(spec.mz)
-            if colorby is not None:
-                # check whether the string is a valid attribute of the spectrum
-                if not hasattr(spec, colorby):
-                    raise ValueError(
-                        f"{colorby} is not a valid attribute of the spectrum {spec_idx}",
-                    )
-                if not isinstance(spec.__dict__[colorby], np.ndarray):
-                    raise ValueError(
-                        f"{colorby} is not a valid attribute of the spectrum {spec_idx}",
-                    )
-                if len(spec.__dict__[colorby]) != len(spec.mz):
-                    raise ValueError(
-                        f"{colorby} is not a valid attribute of the spectrum {spec_idx}",
-                    )
-                else:
-                    mcvalues = spec.__dict__[colorby]
-                    mcvalues[mcvalues < -1] = -1
-                    mcvalues[mcvalues > 1] = 1
-
-                    # normalize the values to be between 0 and 255
-                    mcvalues = (mcvalues + 1) / 2 * 255
-
-                    cm_markers = process_cmap(cmap, ncolors=255, provider=cmap_provider)
-                    # assign colors to the peaks based on the colorby attribute. Set Nans to black
-                    mcolors = [
-                        rgb2hex(cm_markers[int(i * (len(cm_markers) - 1) / 255)])
-                        if not np.isnan(i)
-                        else rgb2hex((0, 0, 0))
-                        for i in mcvalues
-                    ]
-
-            color = colors[spec_idx]
-            mz = spec.mz
-            inty = spec.inty
-            if mz_start is not None:
-                mask = mz >= mz_start
-                mz = mz[mask]
-                inty = inty[mask]
-                mcolors = np.array(mcolors)[mask].tolist()
-            if mz_stop is not None:
-                mask = mz <= mz_stop
-                mz = mz[mask]
-                inty = inty[mask]
-                mcolors = np.array(mcolors)[mask].tolist()
-
-            if len(mz) == 0:
-                print("No peaks in spectrum after trimming")
-                return
-
-            if not spec.centroided:
-                # For profile spectra, adjust the points for line continuity
-                mz_diff = np.diff(mz)
-                new_mzs: list[float] = []
-                new_inty: list[float] = []
-                last_good_step = 1
-                for i in range(len(mz_diff)):
-                    if mz_diff[i] > last_good_step * 4:
-                        new_mzs.append(mz[i] + last_good_step)
-                        new_inty.append(0)
-                        new_mzs.append(mz[i + 1] - last_good_step)
-                        new_inty.append(0)
-                    else:
-                        last_good_step = mz_diff[i]
-                if len(new_mzs) > 0:
-                    new_mzs_array = np.array(new_mzs)
-                    new_inty_array = np.array(new_inty)
-                    mz = np.append(mz, new_mzs_array)
-                    inty = np.append(inty, new_inty_array)
-                    idx = np.argsort(mz)
-                    mz = mz[idx]
-                    inty = inty[idx]
-
-                # Plot profile spectrum as a line
-                p.line(mz, inty, line_color=color, legend_label=label)
+    # Create stacked plots or single plot
+    if stacked:
+        from bokeh.layouts import column
+        plots = []
+        
+        for spec_idx, spec in enumerate(spectra):
+            # Create individual plot for each spectrum
+            if plots:
+                # Sync x and y axes with first plot
+                p = figure(
+                    width=width,
+                    height=height,
+                    title=None,
+                    x_range=plots[0].x_range,
+                    y_range=plots[0].y_range,
+                )
             else:
-                # For centroided spectra, build a data source that includes all available array attributes
-                data = spec.to_dict()
-                # remove all keys whose value does not have the size of mz
-                data = {key: val for key, val in data.items() if isinstance(val, np.ndarray) and val.size == mz.size}
-                data["zeros"] = np.zeros_like(mz)
-                if colorby is not None:
-                    data[colorby] = mcolors
-                source = ColumnDataSource(data)
+                # First plot gets the title
+                p = figure(
+                    width=width,
+                    height=height,
+                    title=title,
+                )
+            
+            try:
+                label = f"Spectrum {spec_idx}"
+                if spec.label is not None:
+                    label = spec.label
+                if labels is not None and len(labels) >= num_plots:
+                    label = labels[spec_idx]
 
-                # seg = p.segment(
-                #     x0="mz",
-                #     y0="zeros",
-                #     x1="mz",
-                #     y1="inty",
-                #     line_color=color,
-                #     legend_label=label,
-                #     source=source,
-                # )
+                mcvalues = None
+                mcolors = ["black"] * len(spec.mz)
                 if colorby is not None:
-                    sc = p.scatter(
-                        x="mz",
-                        y="inty",
-                        size=5,
-                        fill_color=colorby,
-                        line_color=colorby,
-                        legend_label=label,
-                        source=source,
-                    )
+                    if not hasattr(spec, colorby):
+                        raise ValueError(f"{colorby} is not a valid attribute of the spectrum {spec_idx}")
+                    if not isinstance(spec.__dict__[colorby], np.ndarray):
+                        raise ValueError(f"{colorby} is not a valid attribute of the spectrum {spec_idx}")
+                    if len(spec.__dict__[colorby]) != len(spec.mz):
+                        raise ValueError(f"{colorby} is not a valid attribute of the spectrum {spec_idx}")
+                    else:
+                        mcvalues = spec.__dict__[colorby].copy()
+                        mcvalues[mcvalues < -1] = -1
+                        mcvalues[mcvalues > 1] = 1
+                        mcvalues = (mcvalues + 1) / 2 * 255
+                        cm_markers = process_cmap(cmap, ncolors=255, provider=cmap_provider)
+                        mcolors = [
+                            rgb2hex(cm_markers[int(i * (len(cm_markers) - 1) / 255)])
+                            if not np.isnan(i)
+                            else rgb2hex((0, 0, 0))
+                            for i in mcvalues
+                        ]
+
+                color = colors[spec_idx]
+                mz = spec.mz.copy()
+                inty = spec.inty.copy()
+                
+                # Build mask for trimming
+                mask = np.ones(len(mz), dtype=bool)
+                if mz_start is not None:
+                    mask &= mz >= mz_start
+                if mz_stop is not None:
+                    mask &= mz <= mz_stop
+                
+                # Apply mask
+                mz = mz[mask]
+                inty = inty[mask]
+                if len(mcolors) == len(mask):
+                    mcolors = np.array(mcolors)[mask].tolist()
+
+                if len(mz) == 0:
+                    print(f"No peaks in spectrum {spec_idx} after trimming")
+                    continue
+
+                if not spec.centroided:
+                    if not np.all(np.diff(mz) >= 0):
+                        idx = np.argsort(mz)
+                        mz = mz[idx]
+                        inty = inty[idx]
+                    
+                    mz_diff = np.diff(mz)
+                    new_mzs: list[float] = []
+                    new_inty: list[float] = []
+                    last_good_step = 0.01
+                    
+                    for i in range(len(mz_diff)):
+                        if mz_diff[i] > last_good_step * 4:
+                            step = min(last_good_step, mz_diff[i] / 4)
+                            mz_before = min(mz[i] + step, mz[i + 1] - 0.001)
+                            mz_after = max(mz[i + 1] - step, mz[i] + 0.001)
+                            if mz_before < mz[i + 1]:
+                                new_mzs.append(mz_before)
+                                new_inty.append(0)
+                            if mz_after > mz[i] and mz_after < mz[i + 1]:
+                                new_mzs.append(mz_after)
+                                new_inty.append(0)
+                        else:
+                            last_good_step = mz_diff[i]
+                    if len(new_mzs) > 0:
+                        new_mzs_array = np.array(new_mzs)
+                        new_inty_array = np.array(new_inty)
+                        mz = np.append(mz, new_mzs_array)
+                        inty = np.append(inty, new_inty_array)
+                        idx = np.argsort(mz)
+                        mz = mz[idx]
+                        inty = inty[idx]
+
+                    p.line(mz, inty, line_color=color, legend_label=label)
                 else:
-                    sc = p.scatter(
-                        x="mz",
-                        y="inty",
-                        size=3,
-                        fill_color=color,
+                    data = {"mz": mz, "inty": inty}
+                    if ylog:
+                        data["zeros"] = np.ones_like(mz)
+                    else:
+                        data["zeros"] = np.zeros_like(mz)
+                    
+                    for key, val in spec.__dict__.items():
+                        if key not in data and isinstance(val, np.ndarray) and val.size == len(spec.mz):
+                            data[key] = val[mask]
+                    
+                    if colorby is not None:
+                        data[colorby] = mcolors
+                    
+                    source = ColumnDataSource(data)
+
+                    # Plot vertical segments (sticks)
+                    p.segment(
+                        x0="mz",
+                        y0="zeros",
+                        x1="mz",
+                        y1="inty",
                         line_color=color,
                         legend_label=label,
                         source=source,
                     )
-                # Create tooltips for all columns in the data source
-                tooltips = [(k, "@" + k) for k in source.data if k != "zeros"]
-                hover_tool = HoverTool(renderers=[sc], tooltips=tooltips)  # seg
-                p.add_tools(hover_tool)
-                box_zoom_tools = [tool for tool in p.toolbar.tools if isinstance(tool, BoxZoomTool)]
-                if box_zoom_tools:
-                    p.toolbar.active_drag = box_zoom_tools[0]
-        except Exception as e:
-            print(f"Error plotting spectrum {spec_idx}: {e}")
-
-    if colorby is not None:
-        # Create a color mapper using the colormap (cm) with fixed range from -1 to 1
-        color_mapper = LinearColorMapper(palette=cm_markers, low=-1, high=1)
-        if ColorBar is not None:
-            color_bar = ColorBar(
-                color_mapper=color_mapper,
-                ticker=FixedTicker(ticks=[-1, -0.5, 0, 0.5, 1]),
-                location=(0, 0),
-            )
-            p.add_layout(color_bar, "right")
-
-    if ylog:
-        p.y_scale = LogScale()
-        p.yaxis.formatter = LogTickFormatter()
-    else:
-        p.yaxis.formatter = NumeralTickFormatter(format="0.0e0")
-    p.legend.click_policy = "hide"
-
-    p.legend.click_policy = "hide"
-    p.yaxis.formatter = NumeralTickFormatter(format="0.0e0")
-
-    if filename is not None:
-        if filename.endswith(".html"):
-            output_file(filename)
-            save(p)
-        elif filename.endswith(".svg"):
-            p.output_backend = "svg"
-            export_svg(p, filename=filename)
-        elif filename.endswith(".png"):
-            export_png(p, filename=filename)
+                    
+                    # Add scatter points on top
+                    if colorby is not None:
+                        sc = p.scatter(
+                            x="mz",
+                            y="inty",
+                            size=5,
+                            fill_color=colorby,
+                            line_color=colorby,
+                            legend_label=label,
+                            source=source,
+                        )
+                    else:
+                        sc = p.scatter(
+                            x="mz",
+                            y="inty",
+                            size=3,
+                            fill_color=color,
+                            line_color=color,
+                            legend_label=label,
+                            source=source,
+                        )
+                    tooltips = [(k, "@" + k) for k in source.data if k != "zeros"]
+                    hover_tool = HoverTool(renderers=[sc], tooltips=tooltips)
+                    p.add_tools(hover_tool)
+                    box_zoom_tools = [tool for tool in p.toolbar.tools if isinstance(tool, BoxZoomTool)]
+                    if box_zoom_tools:
+                        p.toolbar.active_drag = box_zoom_tools[0]
+            except Exception as e:
+                print(f"Error plotting spectrum {spec_idx}: {e}")
+            
+            # Configure y-axis
+            if ylog:
+                p.y_scale = LogScale()
+                p.yaxis.formatter = LogTickFormatter()
+            else:
+                p.yaxis.formatter = NumeralTickFormatter(format="0.0e0")
+            
+            p.legend.click_policy = "hide"
+            plots.append(p)
+        
+        # Combine all plots in a column layout
+        layout = column(*plots)
+        
+        if filename is not None:
+            if filename.endswith(".html"):
+                output_file(filename)
+                save(layout)
+            elif filename.endswith(".svg"):
+                for plot in plots:
+                    plot.output_backend = "svg"
+                export_svg(layout, filename=filename)
+            elif filename.endswith(".png"):
+                export_png(layout, filename=filename)
+            else:
+                show(layout)
         else:
-            show(p)
+            from bokeh.io import output_notebook, reset_output
+            reset_output()
+            output_notebook()
+            show(layout)
     else:
-        show(p)
+        # Original single plot behavior
+        p = figure(
+            width=width,
+            height=height,
+            title=title,
+        )
+
+        for spec_idx, spec in enumerate(spectra):
+            try:
+                label = f"Spectrum {spec_idx}"
+                if spec.label is not None:
+                    label = spec.label
+                if labels is not None and len(labels) >= num_plots:
+                    label = labels[spec_idx]
+
+                mcvalues = None
+                mcolors = ["black"] * len(spec.mz)
+                if colorby is not None:
+                    if not hasattr(spec, colorby):
+                        raise ValueError(f"{colorby} is not a valid attribute of the spectrum {spec_idx}")
+                    if not isinstance(spec.__dict__[colorby], np.ndarray):
+                        raise ValueError(f"{colorby} is not a valid attribute of the spectrum {spec_idx}")
+                    if len(spec.__dict__[colorby]) != len(spec.mz):
+                        raise ValueError(f"{colorby} is not a valid attribute of the spectrum {spec_idx}")
+                    else:
+                        mcvalues = spec.__dict__[colorby].copy()
+                        mcvalues[mcvalues < -1] = -1
+                        mcvalues[mcvalues > 1] = 1
+                        mcvalues = (mcvalues + 1) / 2 * 255
+                        cm_markers = process_cmap(cmap, ncolors=255, provider=cmap_provider)
+                        mcolors = [
+                            rgb2hex(cm_markers[int(i * (len(cm_markers) - 1) / 255)])
+                            if not np.isnan(i)
+                            else rgb2hex((0, 0, 0))
+                            for i in mcvalues
+                        ]
+
+                color = colors[spec_idx]
+                mz = spec.mz.copy()
+                inty = spec.inty.copy()
+                
+                # Build mask for trimming
+                mask = np.ones(len(mz), dtype=bool)
+                if mz_start is not None:
+                    mask &= mz >= mz_start
+                if mz_stop is not None:
+                    mask &= mz <= mz_stop
+                
+                # Apply mask
+                mz = mz[mask]
+                inty = inty[mask]
+                if len(mcolors) == len(mask):
+                    mcolors = np.array(mcolors)[mask].tolist()
+
+                if len(mz) == 0:
+                    print("No peaks in spectrum after trimming")
+                    return
+
+                if not spec.centroided:
+                    if not np.all(np.diff(mz) >= 0):
+                        idx = np.argsort(mz)
+                        mz = mz[idx]
+                        inty = inty[idx]
+                    
+                    mz_diff = np.diff(mz)
+                    new_mzs: list[float] = []
+                    new_inty: list[float] = []
+                    last_good_step = 0.01
+                    
+                    for i in range(len(mz_diff)):
+                        if mz_diff[i] > last_good_step * 4:
+                            step = min(last_good_step, mz_diff[i] / 4)
+                            mz_before = min(mz[i] + step, mz[i + 1] - 0.001)
+                            mz_after = max(mz[i + 1] - step, mz[i] + 0.001)
+                            if mz_before < mz[i + 1]:
+                                new_mzs.append(mz_before)
+                                new_inty.append(0)
+                            if mz_after > mz[i] and mz_after < mz[i + 1]:
+                                new_mzs.append(mz_after)
+                                new_inty.append(0)
+                        else:
+                            last_good_step = mz_diff[i]
+                    if len(new_mzs) > 0:
+                        new_mzs_array = np.array(new_mzs)
+                        new_inty_array = np.array(new_inty)
+                        mz = np.append(mz, new_mzs_array)
+                        inty = np.append(inty, new_inty_array)
+                        idx = np.argsort(mz)
+                        mz = mz[idx]
+                        inty = inty[idx]
+
+                    p.line(mz, inty, line_color=color, legend_label=label)
+                else:
+                    data = {"mz": mz, "inty": inty}
+                    if ylog:
+                        data["zeros"] = np.ones_like(mz)
+                    else:
+                        data["zeros"] = np.zeros_like(mz)
+                    
+                    for key, val in spec.__dict__.items():
+                        if key not in data and isinstance(val, np.ndarray) and val.size == len(spec.mz):
+                            data[key] = val[mask]
+                    
+                    if colorby is not None:
+                        data[colorby] = mcolors
+                    
+                    source = ColumnDataSource(data)
+
+                    # Plot vertical segments (sticks)
+                    p.segment(
+                        x0="mz",
+                        y0="zeros",
+                        x1="mz",
+                        y1="inty",
+                        line_color=color,
+                        legend_label=label,
+                        source=source,
+                    )
+                    
+                    # Add scatter points on top
+                    if colorby is not None:
+                        sc = p.scatter(
+                            x="mz",
+                            y="inty",
+                            size=5,
+                            fill_color=colorby,
+                            line_color=colorby,
+                            legend_label=label,
+                            source=source,
+                        )
+                    else:
+                        sc = p.scatter(
+                            x="mz",
+                            y="inty",
+                            size=3,
+                            fill_color=color,
+                            line_color=color,
+                            legend_label=label,
+                            source=source,
+                        )
+                    tooltips = [(k, "@" + k) for k in source.data if k != "zeros"]
+                    hover_tool = HoverTool(renderers=[sc], tooltips=tooltips)
+                    p.add_tools(hover_tool)
+                    box_zoom_tools = [tool for tool in p.toolbar.tools if isinstance(tool, BoxZoomTool)]
+                    if box_zoom_tools:
+                        p.toolbar.active_drag = box_zoom_tools[0]
+            except Exception as e:
+                print(f"Error plotting spectrum {spec_idx}: {e}")
+
+        if colorby is not None:
+            color_mapper = LinearColorMapper(palette=cm_markers, low=-1, high=1)
+            if ColorBar is not None:
+                color_bar = ColorBar(
+                    color_mapper=color_mapper,
+                    ticker=FixedTicker(ticks=[-1, -0.5, 0, 0.5, 1]),
+                    location=(0, 0),
+                )
+                p.add_layout(color_bar, "right")
+
+        if ylog:
+            p.y_scale = LogScale()
+            p.yaxis.formatter = LogTickFormatter()
+        else:
+            p.yaxis.formatter = NumeralTickFormatter(format="0.0e0")
+        p.legend.click_policy = "hide"
+
+        if filename is not None:
+            if filename.endswith(".html"):
+                output_file(filename)
+                save(p)
+            elif filename.endswith(".svg"):
+                p.output_backend = "svg"
+                export_svg(p, filename=filename)
+            elif filename.endswith(".png"):
+                export_png(p, filename=filename)
+            else:
+                show(p)
+        else:
+            from bokeh.io import output_notebook, reset_output
+            reset_output()
+            output_notebook()
+            show(p)
 
 
 if __name__ == "__main__":
