@@ -8,7 +8,7 @@ from typing import Optional
 class export_mgf_defaults:
     """Default parameters for exporting study results."""
 
-    filename: str = "features.mgf"
+    filename: str = "consensus.mgf"
     selection: str = "best"  # "best" or "all"
     split_energy: bool = True
     merge: bool = False
