@@ -421,7 +421,7 @@ def export_mgf(self, **kwargs):
             - Individual parameter names and values (see export_defaults for details)
 
     Key Parameters:
-        filename (str): Output MGF file name (default: "features.mgf").
+        filename (str): Output MGF file name (default: "consensus.mgf").
         selection (str): "best" for first scan, "all" for every scan (default: "best").
         split_energy (bool): Process MS2 scans by unique energy (default: True).
         merge (bool): If selection="all", merge MS2 scans into one spectrum (default: False).
@@ -914,8 +914,8 @@ def export_mztab(self, filename: str | None = None, include_mgf=True, **kwargs)
             isotopomer,  # isotopomer formatted according to mzTab-M specification
             safe_str(row.get("mz", "null")),  # exp_mass_to_charge
             safe_str(
-                row.get("adduct_charge_top", "null"),
-            ),  # Use top-ranked adduct charge
+                f"{abs(row.get('adduct_charge_top', 1))}+" if row.get('adduct_charge_top', 1) >= 0 else f"{abs(row.get('adduct_charge_top', 1))}-"
+            ) if row.get("adduct_charge_top") is not None and row.get("adduct_charge_top") != "null" else "null",  # Use top-ranked adduct charge with sign
             safe_str(row.get("rt", "null")),  # retention_time_in_seconds
             safe_str(row.get("retention_time_in_seconds_start", "null")),
             safe_str(row.get("retention_time_in_seconds_end", "null")),
@@ -1066,8 +1066,8 @@ def export_mztab(self, filename: str | None = None, include_mgf=True, **kwargs)
                         adduct_ion,
                         exp_mz,  # experimental m/z from consensus feature
                         safe_str(
-                            consensus_row.get("adduct_charge_top", "1"),
-                        ),  # Use consensus feature's top adduct charge
+                            f"{abs(consensus_row.get('adduct_charge_top', 1))}+" if consensus_row.get('adduct_charge_top', 1) >= 0 else f"{abs(consensus_row.get('adduct_charge_top', 1))}-"
+                        ) if consensus_row.get("adduct_charge_top") is not None and consensus_row.get("adduct_charge_top") != "null" else "1+",  # Use consensus feature's top adduct charge with sign
                         theoretical_mz,  # theoretical m/z from lib_df
                         spectra_ref,
                         id_method,
