@@ -2073,6 +2073,12 @@ def _load_study5(self, filename=None):
             else:
                 self.lib_df = _create_empty_dataframe_from_schema("lib_df", schema)
             pbar.update(1)
+            
+            # Migration: Rename source_id to lib_source for backward compatibility (schema v1.0 -> v1.1)
+            file_schema_version = schema.get("schema_version", "1.0")
+            if "source_id" in self.lib_df.columns and "lib_source" not in self.lib_df.columns:
+                self.logger.info(f"Migrating lib_df from schema v{file_schema_version}: renaming 'source_id' to 'lib_source'")
+                self.lib_df = self.lib_df.rename({"source_id": "lib_source"})
 
             # Load id_df
             pbar.set_description(
