@@ -112,7 +112,7 @@ from masster.study.parameters import set_parameters_property
 from masster.study.save import save, save_consensus, save_samples
 from masster.study.export import export_mgf, export_mztab, export_excel, export_parquet, export_csv
 from masster.study.id import lib_load, identify, get_id, id_reset, lib_reset, lib_compare, lib_select, lib_filter, _get_adducts
-from masster.study.importers import import_oracle
+from masster.study.importers import import_oracle, import_tima
 
 from masster.logger import MassterLogger
 from masster.study.defaults.study_def import study_defaults
@@ -460,8 +460,9 @@ class Study:
     lib_select = lib_select
     lib_filter = lib_filter
 
-    # === Oracle Import Operations ===
+    # === Import Operations ===
     import_oracle = import_oracle
+    import_tima = import_tima
 
     # === Parameter Management ===
     update_history = update_history
