@@ -874,6 +874,12 @@ class Wizard:
             "    s.plot_consensus_2d(cmap='viridis_r')",
             '    return ()',
             '',
+            '@app.cell',
+            'def __(s):',
+            '    # s.lib_load("human")',
+            '    # s.identify()',
+            '    return ()',
+            '',
             'if __name__ == "__main__":',
             "    app.run()",
         ]
